@@ -121,7 +121,7 @@ macro enum(T, syms...)
     end
     basetype = Int32
     typename = T
-    if isa(T, Expr) && T.head == :(::) && length(T.args) == 2 && isa(T.args[1], Symbol)
+    if isa(T, Expr) && T.head === :(::) && length(T.args) == 2 && isa(T.args[1], Symbol)
         typename = T.args[1]
         basetype = Core.eval(__module__, T.args[2])
         if !isa(basetype, DataType) || !(basetype <: Integer) || !isbitstype(basetype)
@@ -137,7 +137,7 @@ macro enum(T, syms...)
     i = zero(basetype)
     hasexpr = false
 
-    if length(syms) == 1 && syms[1] isa Expr && syms[1].head == :block
+    if length(syms) == 1 && syms[1] isa Expr && syms[1].head === :block
         syms = syms[1].args
     end
     for s in syms
@@ -147,7 +147,7 @@ macro enum(T, syms...)
                 throw(ArgumentError("overflow in value \"$s\" of Enum $typename"))
             end
         elseif isa(s, Expr) &&
-               (s.head == :(=) || s.head == :kw) &&
+               (s.head === :(=) || s.head === :kw) &&
                length(s.args) == 2 && isa(s.args[1], Symbol)
             i = Core.eval(__module__, s.args[2]) # allow exprs, e.g. uint128"1"
             if !isa(i, Integer)
