@@ -33,12 +33,14 @@ const CONVERT_NO_UTF_CHECK = 0x00000002 % UInt32
 const CONVERT_POSIX_BASIC = 0x00000004 % UInt32
 const CONVERT_POSIX_EXTENDED = 0x00000008 % UInt32
 const CONVERT_UTF = 0x00000001 % UInt32
+const COPY_MATCHED_SUBJECT = 0x00004000 % UInt32
 const DFA_RESTART = 0x00000040 % UInt32
 const DFA_SHORTEST = 0x00000080 % UInt32
 const DOLLAR_ENDONLY = 0x00000010 % UInt32
 const DOTALL = 0x00000020 % UInt32
 const DUPNAMES = 0x00000040 % UInt32
 const ENDANCHORED = 0x20000000 % UInt32
+const ERROR_ALPHA_ASSERTION_UNKNOWN = 195 % UInt32
 const ERROR_BACKSLASH_C_CALLER_DISABLED = 183 % UInt32
 const ERROR_BACKSLASH_C_LIBRARY_DISABLED = 185 % UInt32
 const ERROR_BACKSLASH_C_SYNTAX = 168 % UInt32
@@ -72,6 +74,7 @@ const ERROR_CLASS_INVALID_RANGE = 150 % UInt32
 const ERROR_CLASS_RANGE_ORDER = 108 % UInt32
 const ERROR_CODE_POINT_TOO_BIG = 134 % UInt32
 const ERROR_CONDITION_ASSERTION_EXPECTED = 128 % UInt32
+const ERROR_CONDITION_ATOMIC_ASSERTION_EXPECTED = 198 % UInt32
 const ERROR_CONVERT_SYNTAX = (-64) % UInt32
 const ERROR_DEFINE_TOO_MANY_BRANCHES = 154 % UInt32
 const ERROR_DEPTHLIMIT = (-53) % UInt32
@@ -79,6 +82,7 @@ const ERROR_DFA_BADRESTART = (-38) % UInt32
 const ERROR_DFA_RECURSE = (-39) % UInt32
 const ERROR_DFA_UCOND = (-40) % UInt32
 const ERROR_DFA_UFUNC = (-41) % UInt32
+const ERROR_DFA_UINVALID_UTF = (-66) % UInt32
 const ERROR_DFA_UITEM = (-42) % UInt32
 const ERROR_DFA_WSSIZE = (-43) % UInt32
 const ERROR_DUPLICATE_SUBPATTERN_NAME = 143 % UInt32
@@ -94,6 +98,7 @@ const ERROR_INTERNAL_BAD_CODE_AUTO_POSSESS = 180 % UInt32
 const ERROR_INTERNAL_BAD_CODE_IN_SKIP = 190 % UInt32
 const ERROR_INTERNAL_BAD_CODE_LOOKBEHINDS = 170 % UInt32
 const ERROR_INTERNAL_CODE_OVERFLOW = 123 % UInt32
+const ERROR_INTERNAL_DUPMATCH = (-65) % UInt32
 const ERROR_INTERNAL_MISSING_SUBPATTERN = 153 % UInt32
 const ERROR_INTERNAL_OVERRAN_WORKSPACE = 152 % UInt32
 const ERROR_INTERNAL_PARSED_OVERFLOW = 163 % UInt32
@@ -102,6 +107,7 @@ const ERROR_INTERNAL_UNEXPECTED_REPEAT = 110 % UInt32
 const ERROR_INTERNAL_UNKNOWN_NEWLINE = 156 % UInt32
 const ERROR_INVALID_AFTER_PARENS_QUERY = 111 % UInt32
 const ERROR_INVALID_HEXADECIMAL = 167 % UInt32
+const ERROR_INVALID_HYPHEN_IN_OPTIONS = 194 % UInt32
 const ERROR_INVALID_OCTAL = 164 % UInt32
 const ERROR_INVALID_SUBPATTERN_NAME = 144 % UInt32
 const ERROR_JIT_BADOPTION = (-45) % UInt32
@@ -145,11 +151,14 @@ const ERROR_QUERY_BARJX_NEST_TOO_DEEP = 184 % UInt32
 const ERROR_RECURSELOOP = (-52) % UInt32
 const ERROR_RECURSIONLIMIT = (-53) % UInt32
 const ERROR_REPMISSINGBRACE = (-58) % UInt32
+const ERROR_SCRIPT_RUN_NOT_AVAILABLE = 196 % UInt32
 const ERROR_SUBPATTERN_NAMES_MISMATCH = 165 % UInt32
 const ERROR_SUBPATTERN_NAME_EXPECTED = 162 % UInt32
 const ERROR_SUBPATTERN_NAME_TOO_LONG = 148 % UInt32
 const ERROR_SUBPATTERN_NUMBER_TOO_BIG = 161 % UInt32
+const ERROR_SUPPORTED_ONLY_IN_UNICODE = 193 % UInt32
 const ERROR_TOOMANYREPLACE = (-61) % UInt32
+const ERROR_TOO_MANY_CAPTURES = 197 % UInt32
 const ERROR_TOO_MANY_CONDITION_BRANCHES = 127 % UInt32
 const ERROR_TOO_MANY_NAMED_SUBPATTERNS = 149 % UInt32
 const ERROR_UCP_IS_DISABLED = 175 % UInt32
@@ -199,7 +208,9 @@ const ERROR_ZERO_RELATIVE_REFERENCE = 126 % UInt32
 const EXTENDED = 0x00000080 % UInt32
 const EXTENDED_MORE = 0x01000000 % UInt32
 const EXTRA_ALLOW_SURROGATE_ESCAPES = 0x00000001 % UInt32
+const EXTRA_ALT_BSUX = 0x00000020 % UInt32
 const EXTRA_BAD_ESCAPE_IS_LITERAL = 0x00000002 % UInt32
+const EXTRA_ESCAPED_CR_IS_LF = 0x00000010 % UInt32
 const EXTRA_MATCH_LINE = 0x00000008 % UInt32
 const EXTRA_MATCH_WORD = 0x00000004 % UInt32
 const FIRSTLINE = 0x00000100 % UInt32
@@ -232,12 +243,14 @@ const INFO_NEWLINE = 20 % UInt32
 const INFO_RECURSIONLIMIT = 21 % UInt32
 const INFO_SIZE = 22 % UInt32
 const JIT_COMPLETE = 0x00000001 % UInt32
+const JIT_INVALID_UTF = 0x00000100 % UInt32
 const JIT_PARTIAL_HARD = 0x00000004 % UInt32
 const JIT_PARTIAL_SOFT = 0x00000002 % UInt32
 const LITERAL = 0x02000000 % UInt32
 const MAJOR = 10 % UInt32
+const MATCH_INVALID_UTF = 0x04000000 % UInt32
 const MATCH_UNSET_BACKREF = 0x00000200 % UInt32
-const MINOR = 31 % UInt32
+const MINOR = 34 % UInt32
 const MULTILINE = 0x00000400 % UInt32
 const NEVER_BACKSLASH_C = 0x00100000 % UInt32
 const NEVER_UCP = 0x00000800 % UInt32
