@@ -41,9 +41,10 @@ function *(transx::Transpose{<:Any,<:StridedVector{T}}, y::StridedVector{T}) whe
 end
 
 # Matrix-vector multiplication
-function (*)(A::StridedMatrix{T}, x::StridedVector{S}) where {T<:BlasFloat,S}
+function (*)(A::StridedMatrix{T}, x::StridedVector{S}) where {T<:BlasFloat,S<:Real}
     TS = promote_op(matprod, T, S)
-    mul!(similar(x, TS, size(A,1)), A, convert(AbstractVector{TS}, x))
+    y = isconcretetype(TS) ? convert(AbstractVector{TS}, x) : x
+    mul!(similar(x, TS, size(A,1)), A, y)
 end
 function (*)(A::AbstractMatrix{T}, x::AbstractVector{S}) where {T,S}
     TS = promote_op(matprod, T, S)
@@ -61,19 +62,24 @@ function *(a::AbstractVector, adjB::Adjoint{<:Any,<:AbstractMatrix})
 end
 (*)(a::AbstractVector, B::AbstractMatrix) = reshape(a,length(a),1)*B
 
-mul!(y::StridedVector{T}, A::StridedVecOrMat{T}, x::StridedVector{T}) where {T<:BlasFloat} = gemv!(y, 'N', A, x)
+@inline mul!(y::StridedVector{T}, A::StridedVecOrMat{T}, x::StridedVector{T},
+             alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasFloat} =
+    gemv!(y, 'N', A, x, alpha, beta)
 # Complex matrix times real vector. Reinterpret the matrix as a real matrix and do real matvec compuation.
 for elty in (Float32,Float64)
     @eval begin
-        function mul!(y::StridedVector{Complex{$elty}}, A::StridedVecOrMat{Complex{$elty}}, x::StridedVector{$elty})
+        @inline function mul!(y::StridedVector{Complex{$elty}}, A::StridedVecOrMat{Complex{$elty}}, x::StridedVector{$elty},
+                              alpha::Union{$elty, Bool}, beta::Union{$elty, Bool})
             Afl = reinterpret($elty,A)
             yfl = reinterpret($elty,y)
-            mul!(yfl,Afl,x)
+            mul!(yfl, Afl, x, alpha, beta)
             return y
         end
     end
 end
-mul!(y::AbstractVector, A::AbstractVecOrMat, x::AbstractVector) = generic_matvecmul!(y, 'N', A, x)
+@inline mul!(y::AbstractVector, A::AbstractVecOrMat, x::AbstractVector,
+             alpha::Number, beta::Number) =
+    generic_matvecmul!(y, 'N', A, x, MulAddMul(alpha, beta))
 
 function *(transA::Transpose{<:Any,<:StridedMatrix{T}}, x::StridedVector{S}) where {T<:BlasFloat,S}
     A = transA.parent
@@ -85,13 +91,15 @@ function *(transA::Transpose{<:Any,<:AbstractMatrix{T}}, x::AbstractVector{S}) w
     TS = promote_op(matprod, T, S)
     mul!(similar(x,TS,size(A,2)), transpose(A), x)
 end
-function mul!(y::StridedVector{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, x::StridedVector{T}) where {T<:BlasFloat}
+@inline function mul!(y::StridedVector{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, x::StridedVector{T},
+                      alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasFloat}
     A = transA.parent
-    return gemv!(y, 'T', A, x)
+    return gemv!(y, 'T', A, x, alpha, beta)
 end
-function mul!(y::AbstractVector, transA::Transpose{<:Any,<:AbstractVecOrMat}, x::AbstractVector)
+@inline function mul!(y::AbstractVector, transA::Transpose{<:Any,<:AbstractVecOrMat}, x::AbstractVector,
+                      alpha::Number, beta::Number)
     A = transA.parent
-    return generic_matvecmul!(y, 'T', A, x)
+    return generic_matvecmul!(y, 'T', A, x, MulAddMul(alpha, beta))
 end
 
 function *(adjA::Adjoint{<:Any,<:StridedMatrix{T}}, x::StridedVector{S}) where {T<:BlasFloat,S}
@@ -105,17 +113,20 @@ function *(adjA::Adjoint{<:Any,<:AbstractMatrix{T}}, x::AbstractVector{S}) where
     mul!(similar(x,TS,size(A,2)), adjoint(A), x)
 end
 
-function mul!(y::StridedVector{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, x::StridedVector{T}) where {T<:BlasReal}
+@inline function mul!(y::StridedVector{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, x::StridedVector{T},
+                      alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasReal}
     A = adjA.parent
-    return mul!(y, transpose(A), x)
+    return mul!(y, transpose(A), x, alpha, beta)
 end
-function mul!(y::StridedVector{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, x::StridedVector{T}) where {T<:BlasComplex}
+@inline function mul!(y::StridedVector{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, x::StridedVector{T},
+                      alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasComplex}
     A = adjA.parent
-    return gemv!(y, 'C', A, x)
+    return gemv!(y, 'C', A, x, alpha, beta)
 end
-function mul!(y::AbstractVector, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, x::AbstractVector)
+@inline function mul!(y::AbstractVector, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, x::AbstractVector,
+                      alpha::Number, beta::Number)
     A = adjA.parent
-    return generic_matvecmul!(y, 'C', A, x)
+    return generic_matvecmul!(y, 'C', A, x, MulAddMul(alpha, beta))
 end
 
 # Vector-Matrix multiplication
@@ -141,15 +152,26 @@ function (*)(A::AbstractMatrix, B::AbstractMatrix)
     TS = promote_op(matprod, eltype(A), eltype(B))
     mul!(similar(B, TS, (size(A,1), size(B,2))), A, B)
 end
-mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T}) where {T<:BlasFloat} = gemm_wrapper!(C, 'N', 'N', A, B)
+
+@inline function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T},
+                      α::Number, β::Number) where {T<:BlasFloat}
+    alpha, beta = promote(α, β, zero(T))
+    if alpha isa T && beta isa T
+        return gemm_wrapper!(C, 'N', 'N', A, B, MulAddMul(alpha, beta))
+    else
+        return generic_matmatmul!(C, 'N', 'N', A, B, MulAddMul(α, β))
+    end
+end
+
 # Complex Matrix times real matrix: We use that it is generally faster to reinterpret the
 # first matrix as a real matrix and carry out real matrix matrix multiply
 for elty in (Float32,Float64)
     @eval begin
-        function mul!(C::StridedMatrix{Complex{$elty}}, A::StridedVecOrMat{Complex{$elty}}, B::StridedVecOrMat{$elty})
+        @inline function mul!(C::StridedMatrix{Complex{$elty}}, A::StridedVecOrMat{Complex{$elty}}, B::StridedVecOrMat{$elty},
+                              alpha::Real, beta::Real)
             Afl = reinterpret($elty, A)
             Cfl = reinterpret($elty, C)
-            mul!(Cfl, Afl, B)
+            mul!(Cfl, Afl, B, alpha, beta)
             return C
         end
     end
@@ -171,13 +193,50 @@ julia> Y
  3.0  3.0
  7.0  7.0
 ```
+
+# Implementation
+For custom matrix and vector types, it is recommended to implement
+5-argument `mul!` rather than implementing 3-argument `mul!` directly
+if possible.
 """
-mul!(C::AbstractMatrix, A::AbstractVecOrMat, B::AbstractVecOrMat) = generic_matmatmul!(C, 'N', 'N', A, B)
+@inline function mul!(C, A, B)
+    return mul!(C, A, B, true, false)
+end
+
+"""
+    mul!(C, A, B, α, β) -> C
+
+Combined inplace matrix-matrix or matrix-vector multiply-add ``A B α + C β``.
+The result is stored in `C` by overwriting it.  Note that `C` must not be
+aliased with either `A` or `B`.
+
+!!! compat "Julia 1.3"
+    Five-argument `mul!` requires at least Julia 1.3.
+
+# Examples
+```jldoctest
+julia> A=[1.0 2.0; 3.0 4.0]; B=[1.0 1.0; 1.0 1.0]; C=[1.0 2.0; 3.0 4.0];
+
+julia> mul!(C, A, B, 100.0, 10.0) === C
+true
+
+julia> C
+2×2 Array{Float64,2}:
+ 310.0  320.0
+ 730.0  740.0
+```
+"""
+@inline mul!(C::AbstractMatrix, A::AbstractVecOrMat, B::AbstractVecOrMat,
+             alpha::Number, beta::Number) =
+    generic_matmatmul!(C, 'N', 'N', A, B, MulAddMul(alpha, beta))
 
 """
     rmul!(A, B)
 
 Calculate the matrix-matrix product ``AB``, overwriting `A`, and return the result.
+Here, `B` must be of special matrix type, like, e.g., [`Diagonal`](@ref),
+[`UpperTriangular`](@ref) or [`LowerTriangular`](@ref), or of some orthogonal type,
+see [`QR`](@ref).
 
 # Examples
 ```jldoctest
@@ -191,6 +250,15 @@ julia> A
 2×2 Array{Int64,2}:
  0  3
  1  2
+
+julia> A = [1.0 2.0; 3.0 4.0];
+
+julia> F = qr([0 1; -1 0]);
+
+julia> rmul!(A, F.Q)
+2×2 Array{Float64,2}:
+ 2.0  1.0
+ 4.0  3.0
 ```
 """
 rmul!(A, B)
@@ -199,6 +267,9 @@ rmul!(A, B)
     lmul!(A, B)
 
 Calculate the matrix-matrix product ``AB``, overwriting `B`, and return the result.
+Here, `A` must be of special matrix type, like, e.g., [`Diagonal`](@ref),
+[`UpperTriangular`](@ref) or [`LowerTriangular`](@ref), or of some orthogonal type,
+see [`QR`](@ref).
 
 # Examples
 ```jldoctest
@@ -212,129 +283,159 @@ julia> B
 2×2 Array{Int64,2}:
  2  1
  3  0
+
+julia> B = [1.0 2.0; 3.0 4.0];
+
+julia> F = qr([0 1; -1 0]);
+
+julia> lmul!(F.Q, B)
+2×2 Array{Float64,2}:
+ 3.0  4.0
+ 1.0  2.0
 ```
 """
 lmul!(A, B)
 
-function mul!(C::StridedMatrix{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, B::StridedVecOrMat{T}) where {T<:BlasFloat}
+@inline function mul!(C::StridedMatrix{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, B::StridedVecOrMat{T},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasFloat}
     A = transA.parent
     if A===B
-        return syrk_wrapper!(C, 'T', A)
+        return syrk_wrapper!(C, 'T', A, MulAddMul(alpha, beta))
     else
-        return gemm_wrapper!(C, 'T', 'N', A, B)
+        return gemm_wrapper!(C, 'T', 'N', A, B, MulAddMul(alpha, beta))
     end
 end
-function mul!(C::AbstractMatrix, transA::Transpose{<:Any,<:AbstractVecOrMat}, B::AbstractVecOrMat)
+@inline function mul!(C::AbstractMatrix, transA::Transpose{<:Any,<:AbstractVecOrMat}, B::AbstractVecOrMat,
+                 alpha::Number, beta::Number)
     A = transA.parent
-    return generic_matmatmul!(C, 'T', 'N', A, B)
+    return generic_matmatmul!(C, 'T', 'N', A, B, MulAddMul(alpha, beta))
 end
 
-function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, transB::Transpose{<:Any,<:StridedVecOrMat{T}}) where {T<:BlasFloat}
+@inline function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, transB::Transpose{<:Any,<:StridedVecOrMat{T}},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasFloat}
     B = transB.parent
     if A===B
-        return syrk_wrapper!(C, 'N', A)
+        return syrk_wrapper!(C, 'N', A, MulAddMul(alpha, beta))
     else
-        return gemm_wrapper!(C, 'N', 'T', A, B)
+        return gemm_wrapper!(C, 'N', 'T', A, B, MulAddMul(alpha, beta))
     end
 end
 # Complex matrix times transposed real matrix. Reinterpret the first matrix to real for efficiency.
 for elty in (Float32,Float64)
     @eval begin
-        function mul!(C::StridedMatrix{Complex{$elty}}, A::StridedVecOrMat{Complex{$elty}}, transB::Transpose{<:Any,<:StridedVecOrMat{$elty}})
+        @inline function mul!(C::StridedMatrix{Complex{$elty}}, A::StridedVecOrMat{Complex{$elty}}, transB::Transpose{<:Any,<:StridedVecOrMat{$elty}},
+                         alpha::Union{$elty, Bool}, beta::Union{$elty, Bool})
             Afl = reinterpret($elty, A)
             Cfl = reinterpret($elty, C)
-            mul!(Cfl,Afl,transB)
+            mul!(Cfl, Afl, transB, alpha, beta)
             return C
         end
     end
 end
 # collapsing the following two defs with C::AbstractVecOrMat yields ambiguities
-mul!(C::AbstractVector, A::AbstractVecOrMat, transB::Transpose{<:Any,<:AbstractVecOrMat}) =
-    generic_matmatmul!(C, 'N', 'T', A, transB.parent)
-mul!(C::AbstractMatrix, A::AbstractVecOrMat, transB::Transpose{<:Any,<:AbstractVecOrMat}) =
-    generic_matmatmul!(C, 'N', 'T', A, transB.parent)
+@inline mul!(C::AbstractVector, A::AbstractVecOrMat, transB::Transpose{<:Any,<:AbstractVecOrMat},
+             alpha::Number, beta::Number) =
+    generic_matmatmul!(C, 'N', 'T', A, transB.parent, MulAddMul(alpha, beta))
+@inline mul!(C::AbstractMatrix, A::AbstractVecOrMat, transB::Transpose{<:Any,<:AbstractVecOrMat},
+             alpha::Number, beta::Number) =
+    generic_matmatmul!(C, 'N', 'T', A, transB.parent, MulAddMul(alpha, beta))
 
-function mul!(C::StridedMatrix{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, transB::Transpose{<:Any,<:StridedVecOrMat{T}}) where {T<:BlasFloat}
+@inline function mul!(C::StridedMatrix{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, transB::Transpose{<:Any,<:StridedVecOrMat{T}},
+                 alpha::Number, beta::Number) where {T<:BlasFloat}
     A = transA.parent
     B = transB.parent
-    return gemm_wrapper!(C, 'T', 'T', A, B)
+    return gemm_wrapper!(C, 'T', 'T', A, B, MulAddMul(alpha, beta))
 end
-function mul!(C::AbstractMatrix, transA::Transpose{<:Any,<:AbstractVecOrMat}, transB::Transpose{<:Any,<:AbstractVecOrMat})
+@inline function mul!(C::AbstractMatrix, transA::Transpose{<:Any,<:AbstractVecOrMat}, transB::Transpose{<:Any,<:AbstractVecOrMat},
+                 alpha::Number, beta::Number)
     A = transA.parent
     B = transB.parent
-    return generic_matmatmul!(C, 'T', 'T', A, B)
+    return generic_matmatmul!(C, 'T', 'T', A, B, MulAddMul(alpha, beta))
 end
 
-function mul!(C::StridedMatrix{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, transB::Adjoint{<:Any,<:StridedVecOrMat{T}}) where {T<:BlasFloat}
+@inline function mul!(C::StridedMatrix{T}, transA::Transpose{<:Any,<:StridedVecOrMat{T}}, transB::Adjoint{<:Any,<:StridedVecOrMat{T}},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasFloat}
     A = transA.parent
     B = transB.parent
-    return gemm_wrapper!(C, 'T', 'C', A, B)
+    return gemm_wrapper!(C, 'T', 'C', A, B, MulAddMul(alpha, beta))
 end
-function mul!(C::AbstractMatrix, transA::Transpose{<:Any,<:AbstractVecOrMat}, transB::Adjoint{<:Any,<:AbstractVecOrMat})
+@inline function mul!(C::AbstractMatrix, transA::Transpose{<:Any,<:AbstractVecOrMat}, transB::Adjoint{<:Any,<:AbstractVecOrMat},
+                 alpha::Number, beta::Number)
     A = transA.parent
     B = transB.parent
-    return generic_matmatmul!(C, 'T', 'C', A, B)
+    return generic_matmatmul!(C, 'T', 'C', A, B, MulAddMul(alpha, beta))
 end
 
-function mul!(C::StridedMatrix{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, B::StridedVecOrMat{T}) where {T<:BlasReal}
+@inline function mul!(C::StridedMatrix{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, B::StridedVecOrMat{T},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasReal}
     A = adjA.parent
-    return mul!(C, transpose(A), B)
+    return mul!(C, transpose(A), B, alpha, beta)
 end
-function mul!(C::StridedMatrix{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, B::StridedVecOrMat{T}) where {T<:BlasComplex}
+@inline function mul!(C::StridedMatrix{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, B::StridedVecOrMat{T},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasComplex}
     A = adjA.parent
     if A===B
-        return herk_wrapper!(C,'C',A)
+        return herk_wrapper!(C, 'C', A, MulAddMul(alpha, beta))
     else
-        return gemm_wrapper!(C,'C', 'N', A, B)
+        return gemm_wrapper!(C, 'C', 'N', A, B, MulAddMul(alpha, beta))
     end
 end
-function mul!(C::AbstractMatrix, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, B::AbstractVecOrMat)
+@inline function mul!(C::AbstractMatrix, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, B::AbstractVecOrMat,
+                 alpha::Number, beta::Number)
     A = adjA.parent
-    return generic_matmatmul!(C, 'C', 'N', A, B)
+    return generic_matmatmul!(C, 'C', 'N', A, B, MulAddMul(alpha, beta))
 end
 
-function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, adjB::Adjoint{<:Any,<:StridedVecOrMat{<:BlasReal}}) where {T<:BlasFloat}
+@inline function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, adjB::Adjoint{<:Any,<:StridedVecOrMat{<:BlasReal}},
+                 alpha::Number, beta::Number) where {T<:BlasFloat}
     B = adjB.parent
-    return mul!(C, A, transpose(B))
+    return mul!(C, A, transpose(B), alpha, beta)
 end
-function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, adjB::Adjoint{<:Any,<:StridedVecOrMat{T}}) where {T<:BlasComplex}
+@inline function mul!(C::StridedMatrix{T}, A::StridedVecOrMat{T}, adjB::Adjoint{<:Any,<:StridedVecOrMat{T}},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasComplex}
     B = adjB.parent
     if A === B
-        return herk_wrapper!(C, 'N', A)
+        return herk_wrapper!(C, 'N', A, MulAddMul(alpha, beta))
     else
-        return gemm_wrapper!(C, 'N', 'C', A, B)
+        return gemm_wrapper!(C, 'N', 'C', A, B, MulAddMul(alpha, beta))
     end
 end
-function mul!(C::AbstractMatrix, A::AbstractVecOrMat, adjB::Adjoint{<:Any,<:AbstractVecOrMat})
+@inline function mul!(C::AbstractMatrix, A::AbstractVecOrMat, adjB::Adjoint{<:Any,<:AbstractVecOrMat},
+                 alpha::Number, beta::Number)
     B = adjB.parent
-    return generic_matmatmul!(C, 'N', 'C', A, B)
+    return generic_matmatmul!(C, 'N', 'C', A, B, MulAddMul(alpha, beta))
 end
 
-function mul!(C::StridedMatrix{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, adjB::Adjoint{<:Any,<:StridedVecOrMat{T}}) where {T<:BlasFloat}
+@inline function mul!(C::StridedMatrix{T}, adjA::Adjoint{<:Any,<:StridedVecOrMat{T}}, adjB::Adjoint{<:Any,<:StridedVecOrMat{T}},
+                 alpha::Union{T, Bool}, beta::Union{T, Bool}) where {T<:BlasFloat}
     A = adjA.parent
     B = adjB.parent
-    return gemm_wrapper!(C, 'C', 'C', A, B)
+    return gemm_wrapper!(C, 'C', 'C', A, B, MulAddMul(alpha, beta))
 end
-function mul!(C::AbstractMatrix, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, adjB::Adjoint{<:Any,<:AbstractVecOrMat})
+@inline function mul!(C::AbstractMatrix, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, adjB::Adjoint{<:Any,<:AbstractVecOrMat},
+                 alpha::Number, beta::Number)
     A = adjA.parent
     B = adjB.parent
-    return generic_matmatmul!(C, 'C', 'C', A, B)
+    return generic_matmatmul!(C, 'C', 'C', A, B, MulAddMul(alpha, beta))
 end
-function mul!(C::AbstractMatrix, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, transB::Transpose{<:Any,<:AbstractVecOrMat})
+@inline function mul!(C::AbstractMatrix, adjA::Adjoint{<:Any,<:AbstractVecOrMat}, transB::Transpose{<:Any,<:AbstractVecOrMat},
+                 alpha::Number, beta::Number)
     A = adjA.parent
     B = transB.parent
-    return generic_matmatmul!(C, 'C', 'T', A, B)
+    return generic_matmatmul!(C, 'C', 'T', A, B, MulAddMul(alpha, beta))
 end
 # Supporting functions for matrix multiplication
 
-function copytri!(A::AbstractMatrix, uplo::AbstractChar, conjugate::Bool=false)
+# copy transposed(adjoint) of upper(lower) side-digonals. Optionally include diagonal.
+@inline function copytri!(A::AbstractMatrix, uplo::AbstractChar, conjugate::Bool=false, diag::Bool=false)
     n = checksquare(A)
+    off = diag ? 0 : 1
     if uplo == 'U'
-        for i = 1:(n-1), j = (i+1):n
+        for i = 1:n, j = (i+off):n
             A[j,i] = conjugate ? adjoint(A[i,j]) : transpose(A[i,j])
         end
     elseif uplo == 'L'
-        for i = 1:(n-1), j = (i+1):n
+        for i = 1:n, j = (i+off):n
             A[i,j] = conjugate ? adjoint(A[j,i]) : transpose(A[j,i])
         end
     else
@@ -343,7 +444,8 @@ function copytri!(A::AbstractMatrix, uplo::AbstractChar, conjugate::Bool=false)
     A
 end
 
-function gemv!(y::StridedVector{T}, tA::AbstractChar, A::StridedVecOrMat{T}, x::StridedVector{T}) where T<:BlasFloat
+function gemv!(y::StridedVector{T}, tA::AbstractChar, A::StridedVecOrMat{T}, x::StridedVector{T},
+               alpha::Union{T, Bool} = true, beta::Union{T, Bool} = false) where T<:BlasFloat
     mA, nA = lapack_size(tA, A)
     if nA != length(x)
         throw(DimensionMismatch("second dimension of A, $nA, does not match length of x, $(length(x))"))
@@ -355,13 +457,16 @@ function gemv!(y::StridedVector{T}, tA::AbstractChar, A::StridedVecOrMat{T}, x::
         return y
     end
     if nA == 0
-        return fill!(y,0)
+        return _rmul_or_fill!(y, beta)
     end
-    stride(A, 1) == 1 && stride(A, 2) >= size(A, 1) && return BLAS.gemv!(tA, one(T), A, x, zero(T), y)
-    return generic_matvecmul!(y, tA, A, x)
+    if stride(A, 1) == 1 && stride(A, 2) >= size(A, 1)
+        return BLAS.gemv!(tA, alpha, A, x, beta, y)
+    end
+    return generic_matvecmul!(y, tA, A, x, MulAddMul(alpha, beta))
 end
 
-function syrk_wrapper!(C::StridedMatrix{T}, tA::AbstractChar, A::StridedVecOrMat{T}) where T<:BlasFloat
+function syrk_wrapper!(C::StridedMatrix{T}, tA::AbstractChar, A::StridedVecOrMat{T},
+                       _add::MulAddMul = MulAddMul()) where T<:BlasFloat
     nC = checksquare(C)
     if tA == 'T'
         (nA, mA) = size(A,1), size(A,2)
@@ -373,23 +478,24 @@ function syrk_wrapper!(C::StridedMatrix{T}, tA::AbstractChar, A::StridedVecOrMat
     if nC != mA
         throw(DimensionMismatch("output matrix has size: $(nC), but should have size $(mA)"))
     end
-    if mA == 0 || nA == 0
-        return fill!(C,0)
+    if mA == 0 || nA == 0 || iszero(_add.alpha)
+        return _rmul_or_fill!(C, _add.beta)
     end
     if mA == 2 && nA == 2
-        return matmul2x2!(C,tA,tAt,A,A)
+        return matmul2x2!(C, tA, tAt, A, A, _add)
     end
     if mA == 3 && nA == 3
-        return matmul3x3!(C,tA,tAt,A,A)
+        return matmul3x3!(C, tA, tAt, A, A, _add)
     end
 
     if stride(A, 1) == stride(C, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(C, 2) >= size(C, 1)
-        return copytri!(BLAS.syrk!('U', tA, one(T), A, zero(T), C), 'U')
+        return copytri!(BLAS.syrk!('U', tA, _add.alpha, A, _add.beta, C), 'U')
     end
-    return generic_matmatmul!(C, tA, tAt, A, A)
+    return generic_matmatmul!(C, tA, tAt, A, A, _add)
 end
 
-function herk_wrapper!(C::Union{StridedMatrix{T}, StridedMatrix{Complex{T}}}, tA::AbstractChar, A::Union{StridedVecOrMat{T}, StridedVecOrMat{Complex{T}}}) where T<:BlasReal
+function herk_wrapper!(C::Union{StridedMatrix{T}, StridedMatrix{Complex{T}}}, tA::AbstractChar, A::Union{StridedVecOrMat{T}, StridedVecOrMat{Complex{T}}},
+                       _add::MulAddMul = MulAddMul()) where T<:BlasReal
     nC = checksquare(C)
     if tA == 'C'
         (nA, mA) = size(A,1), size(A,2)
@@ -402,22 +508,22 @@ function herk_wrapper!(C::Union{StridedMatrix{T}, StridedMatrix{Complex{T}}}, tA
         throw(DimensionMismatch("output matrix has size: $(nC), but should have size $(mA)"))
     end
     if mA == 0 || nA == 0
-        return fill!(C,0)
+        return _rmul_or_fill!(C, _add.beta)
     end
     if mA == 2 && nA == 2
-        return matmul2x2!(C,tA,tAt,A,A)
+        return matmul2x2!(C, tA, tAt, A, A, _add)
     end
     if mA == 3 && nA == 3
-        return matmul3x3!(C,tA,tAt,A,A)
+        return matmul3x3!(C, tA, tAt, A, A, _add)
     end
 
     # Result array does not need to be initialized as long as beta==0
     #    C = Matrix{T}(undef, mA, mA)
 
     if stride(A, 1) == stride(C, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(C, 2) >= size(C, 1)
-        return copytri!(BLAS.herk!('U', tA, one(T), A, zero(T), C), 'U', true)
+        return copytri!(BLAS.herk!('U', tA, _add.alpha, A, _add.beta, C), 'U', true)
     end
-    return generic_matmatmul!(C,tA, tAt, A, A)
+    return generic_matmatmul!(C, tA, tAt, A, A, _add)
 end
 
 function gemm_wrapper(tA::AbstractChar, tB::AbstractChar,
@@ -430,8 +536,8 @@ function gemm_wrapper(tA::AbstractChar, tB::AbstractChar,
 end
 
 function gemm_wrapper!(C::StridedVecOrMat{T}, tA::AbstractChar, tB::AbstractChar,
-                       A::StridedVecOrMat{T},
-                       B::StridedVecOrMat{T}) where T<:BlasFloat
+                       A::StridedVecOrMat{T}, B::StridedVecOrMat{T},
+                       _add::MulAddMul = MulAddMul()) where T<:BlasFloat
     mA, nA = lapack_size(tA, A)
     mB, nB = lapack_size(tB, B)
 
@@ -443,24 +549,24 @@ function gemm_wrapper!(C::StridedVecOrMat{T}, tA::AbstractChar, tB::AbstractChar
         throw(ArgumentError("output matrix must not be aliased with input matrix"))
     end
 
-    if mA == 0 || nA == 0 || nB == 0
+    if mA == 0 || nA == 0 || nB == 0 || iszero(_add.alpha)
         if size(C) != (mA, nB)
             throw(DimensionMismatch("C has dimensions $(size(C)), should have ($mA,$nB)"))
         end
-        return fill!(C,0)
+        return _rmul_or_fill!(C, _add.beta)
     end
 
     if mA == 2 && nA == 2 && nB == 2
-        return matmul2x2!(C,tA,tB,A,B)
+        return matmul2x2!(C, tA, tB, A, B, _add)
     end
     if mA == 3 && nA == 3 && nB == 3
-        return matmul3x3!(C,tA,tB,A,B)
+        return matmul3x3!(C, tA, tB, A, B, _add)
     end
 
     if stride(A, 1) == stride(B, 1) == stride(C, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(B, 2) >= size(B, 1) && stride(C, 2) >= size(C, 1)
-        return BLAS.gemm!(tA, tB, one(T), A, B, zero(T), C)
+        return BLAS.gemm!(tA, tB, _add.alpha, A, B, _add.beta, C)
     end
-    generic_matmatmul!(C, tA, tB, A, B)
+    generic_matmatmul!(C, tA, tB, A, B, _add)
 end
 
 # blas.jl defines matmul for floats; other integer and mixed precision
@@ -494,8 +600,9 @@ end
 # NOTE: the generic version is also called as fallback for
 #       strides != 1 cases
 
-function generic_matvecmul!(C::AbstractVector{R}, tA, A::AbstractVecOrMat, B::AbstractVector) where R
-    @assert !has_offset_axes(C, A, B)
+function generic_matvecmul!(C::AbstractVector{R}, tA, A::AbstractVecOrMat, B::AbstractVector,
+                            _add::MulAddMul = MulAddMul()) where R
+    require_one_based_indexing(C, A, B)
     mB = length(B)
     mA, nA = lapack_size(tA, A)
     if mB != nA
@@ -519,7 +626,7 @@ function generic_matvecmul!(C::AbstractVector{R}, tA, A::AbstractVecOrMat, B::Ab
             for i = 1:nA
                 s += transpose(A[aoffs+i]) * B[i]
             end
-            C[k] = s
+            _modify!(_add, s, C, k)
         end
     elseif tA == 'C'
         for k = 1:mA
@@ -532,11 +639,13 @@ function generic_matvecmul!(C::AbstractVector{R}, tA, A::AbstractVecOrMat, B::Ab
             for i = 1:nA
                 s += A[aoffs + i]'B[i]
             end
-            C[k] = s
+            _modify!(_add, s, C, k)
         end
     else # tA == 'N'
         for i = 1:mA
-            if mB == 0
+            if !iszero(_add.beta)
+                C[i] *= _add.beta
+            elseif mB == 0
                 C[i] = zero(R)
             else
                 C[i] = zero(A[i]*B[1] + A[i]*B[1])
@@ -544,7 +653,7 @@ function generic_matvecmul!(C::AbstractVector{R}, tA, A::AbstractVecOrMat, B::Ab
         end
         for k = 1:mB
             aoffs = (k-1)*Astride
-            b = B[k]
+            b = _add(B[k], 0)
             for i = 1:mA
                 C[i] += A[aoffs + i] * b
             end
@@ -567,24 +676,30 @@ const Abuf = [Vector{UInt8}(undef, tilebufsize)]
 const Bbuf = [Vector{UInt8}(undef, tilebufsize)]
 const Cbuf = [Vector{UInt8}(undef, tilebufsize)]
 
-function generic_matmatmul!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMatrix)
+function generic_matmatmul!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMatrix,
+                            _add::MulAddMul = MulAddMul())
     mA, nA = lapack_size(tA, A)
     mB, nB = lapack_size(tB, B)
     mC, nC = size(C)
 
+    if iszero(_add.alpha)
+        return _rmul_or_fill!(C, _add.beta)
+    end
     if mA == nA == mB == nB == mC == nC == 2
-        return matmul2x2!(C, tA, tB, A, B)
+        return matmul2x2!(C, tA, tB, A, B, _add)
     end
     if mA == nA == mB == nB == mC == nC == 3
-        return matmul3x3!(C, tA, tB, A, B)
+        return matmul3x3!(C, tA, tB, A, B, _add)
     end
-    _generic_matmatmul!(C, tA, tB, A, B)
+    _generic_matmatmul!(C, tA, tB, A, B, _add)
 end
 
-generic_matmatmul!(C::AbstractVecOrMat, tA, tB, A::AbstractVecOrMat, B::AbstractVecOrMat) = _generic_matmatmul!(C, tA, tB, A, B)
+generic_matmatmul!(C::AbstractVecOrMat, tA, tB, A::AbstractVecOrMat, B::AbstractVecOrMat, _add::MulAddMul) =
+    _generic_matmatmul!(C, tA, tB, A, B, _add)
 
-function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat{T}, B::AbstractVecOrMat{S}) where {T,S,R}
-    @assert !has_offset_axes(C, A, B)
+function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat{T}, B::AbstractVecOrMat{S},
+                             _add::MulAddMul) where {T,S,R}
+    require_one_based_indexing(C, A, B)
     mA, nA = lapack_size(tA, A)
     mB, nB = lapack_size(tB, B)
     if mB != nA
@@ -593,8 +708,8 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
     if size(C,1) != mA || size(C,2) != nB
         throw(DimensionMismatch("result C has dimensions $(size(C)), needs ($mA,$nB)"))
     end
-    if isempty(A) || isempty(B)
-        return fill!(C, zero(R))
+    if isempty(A) || isempty(B) || iszero(_add.alpha)
+        return _rmul_or_fill!(C, _add.beta)
     end
 
     tile_size = 0
@@ -622,7 +737,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         s += Atile[aoff+k] * Btile[boff+k]
                     end
-                    C[i,j] = s
+                    _modify!(_add, s, C, (i,j))
                 end
             end
         else
@@ -652,7 +767,11 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                             end
                         end
                     end
-                    copyto!(C, ib:ilim, jb:jlim, Ctile, 1:ilen, 1:jlen)
+                    if isone(_add.alpha) && iszero(_add.beta)
+                        copyto!(C, ib:ilim, jb:jlim, Ctile, 1:ilen, 1:jlen)
+                    else
+                        C[ib:ilim, jb:jlim] .= @views _add.(Ctile[1:ilen, 1:jlen], C[ib:ilim, jb:jlim])
+                    end
                 end
             end
         end
@@ -666,7 +785,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += A[i, k]*B[k, j]
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             elseif tB == 'T'
                 for i = 1:mA, j = 1:nB
@@ -675,7 +794,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += A[i, k] * transpose(B[j, k])
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             else
                 for i = 1:mA, j = 1:nB
@@ -684,7 +803,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += A[i, k]*B[j, k]'
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             end
         elseif tA == 'T'
@@ -695,7 +814,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += transpose(A[k, i]) * B[k, j]
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             elseif tB == 'T'
                 for i = 1:mA, j = 1:nB
@@ -704,7 +823,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += transpose(A[k, i]) * transpose(B[j, k])
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             else
                 for i = 1:mA, j = 1:nB
@@ -713,7 +832,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += transpose(A[k, i]) * adjoint(B[j, k])
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             end
         else
@@ -724,7 +843,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += A[k, i]'B[k, j]
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             elseif tB == 'T'
                 for i = 1:mA, j = 1:nB
@@ -733,7 +852,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += adjoint(A[k, i]) * transpose(B[j, k])
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             else
                 for i = 1:mA, j = 1:nB
@@ -742,7 +861,7 @@ function _generic_matmatmul!(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat
                     for k = 1:nA
                         Ctmp += A[k, i]'B[j, k]'
                     end
-                    C[i,j] = Ctmp
+                    _modify!(_add, Ctmp, C, (i,j))
                 end
             end
         end
@@ -757,8 +876,9 @@ function matmul2x2(tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S}) where {T,
     matmul2x2!(similar(B, promote_op(matprod, T, S), 2, 2), tA, tB, A, B)
 end
 
-function matmul2x2!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMatrix)
-    @assert !has_offset_axes(C, A, B)
+function matmul2x2!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMatrix,
+                    _add::MulAddMul = MulAddMul())
+    require_one_based_indexing(C, A, B)
     if !(size(A) == size(B) == size(C) == (2,2))
         throw(DimensionMismatch("A has size $(size(A)), B has size $(size(B)), C has size $(size(C))"))
     end
@@ -786,10 +906,10 @@ function matmul2x2!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMat
         B11 = B[1,1]; B12 = B[1,2];
         B21 = B[2,1]; B22 = B[2,2]
     end
-    C[1,1] = A11*B11 + A12*B21
-    C[1,2] = A11*B12 + A12*B22
-    C[2,1] = A21*B11 + A22*B21
-    C[2,2] = A21*B12 + A22*B22
+    _modify!(_add, A11*B11 + A12*B21, C, (1,1))
+    _modify!(_add, A11*B12 + A12*B22, C, (1,2))
+    _modify!(_add, A21*B11 + A22*B21, C, (2,1))
+    _modify!(_add, A21*B12 + A22*B22, C, (2,2))
     end # inbounds
     C
 end
@@ -799,8 +919,9 @@ function matmul3x3(tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S}) where {T,
     matmul3x3!(similar(B, promote_op(matprod, T, S), 3, 3), tA, tB, A, B)
 end
 
-function matmul3x3!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMatrix)
-    @assert !has_offset_axes(C, A, B)
+function matmul3x3!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMatrix,
+                    _add::MulAddMul = MulAddMul())
+    require_one_based_indexing(C, A, B)
     if !(size(A) == size(B) == size(C) == (3,3))
         throw(DimensionMismatch("A has size $(size(A)), B has size $(size(B)), C has size $(size(C))"))
     end
@@ -837,17 +958,17 @@ function matmul3x3!(C::AbstractMatrix, tA, tB, A::AbstractMatrix, B::AbstractMat
         B31 = B[3,1]; B32 = B[3,2]; B33 = B[3,3]
     end
 
-    C[1,1] = A11*B11 + A12*B21 + A13*B31
-    C[1,2] = A11*B12 + A12*B22 + A13*B32
-    C[1,3] = A11*B13 + A12*B23 + A13*B33
+    _modify!(_add, A11*B11 + A12*B21 + A13*B31, C, (1,1))
+    _modify!(_add, A11*B12 + A12*B22 + A13*B32, C, (1,2))
+    _modify!(_add, A11*B13 + A12*B23 + A13*B33, C, (1,3))
 
-    C[2,1] = A21*B11 + A22*B21 + A23*B31
-    C[2,2] = A21*B12 + A22*B22 + A23*B32
-    C[2,3] = A21*B13 + A22*B23 + A23*B33
+    _modify!(_add, A21*B11 + A22*B21 + A23*B31, C, (2,1))
+    _modify!(_add, A21*B12 + A22*B22 + A23*B32, C, (2,2))
+    _modify!(_add, A21*B13 + A22*B23 + A23*B33, C, (2,3))
 
-    C[3,1] = A31*B11 + A32*B21 + A33*B31
-    C[3,2] = A31*B12 + A32*B22 + A33*B32
-    C[3,3] = A31*B13 + A32*B23 + A33*B33
+    _modify!(_add, A31*B11 + A32*B21 + A33*B31, C, (3,1))
+    _modify!(_add, A31*B12 + A32*B22 + A33*B32, C, (3,2))
+    _modify!(_add, A31*B13 + A32*B23 + A33*B33, C, (3,3))
     end # inbounds
     C
 end
