@@ -6,6 +6,7 @@ import Random: randstring
 import LibGit2
 using Test
 using UUIDs
+using Dates
 
 using Pkg
 using Pkg.Types
@@ -94,6 +95,32 @@ import Pkg.Types: semver_spec, VersionSpec
     @test !(Pkg.Types.isjoinable(Pkg.Types.VersionBound((1,5)), Pkg.Types.VersionBound((1,6,0))))
 end
 
+@testset "compress_versions()" begin
+    # Test exact version matching
+    vs = [v"1.1.0", v"1.1.1", v"1.1.2"]
+    @test Pkg.Types.compress_versions(vs, [vs[2]]) == Pkg.Types.VersionSpec("1.1.1")
+    
+    # Test holes
+    vs = [v"1.1.0", v"1.1.1", v"1.1.4"]
+    @test Pkg.Types.compress_versions(vs, [vs[2]]) == Pkg.Types.VersionSpec("1.1.1")
+    
+    # Test patch variation with length(subset) > 1
+    vs = [v"1.1.0", v"1.1.1", v"1.1.2", v"1.1.3", v"1.2.0"]
+    @test Pkg.Types.compress_versions(vs, [vs[2], vs[3]]) == Pkg.Types.VersionSpec("1.1.1-1.1.2")
+    
+    # Test minor variation
+    vs = [v"1.1.0", v"1.1.1", v"1.2.0"]
+    @test Pkg.Types.compress_versions(vs, [vs[2]]) == Pkg.Types.VersionSpec("1.1.1-1.1")
+   
+    # Test major variation
+    vs = [v"1.1.0", v"1.1.1", v"1.2.0", v"2.0.0"]
+    @test Pkg.Types.compress_versions(vs, [vs[2], vs[3]]) == Pkg.Types.VersionSpec("1.1.1-1")
+
+    # Test build numbers and prerelease values are ignored
+    vs = [v"1.1.0-alpha", v"1.1.0+0", v"1.1.0+1"]
+    @test Pkg.Types.compress_versions(vs, [vs[2]]) == Pkg.Types.VersionSpec("1")
+end
+
 # TODO: Should rewrite these tests not to rely on internals like field names
 @testset "union, isjoinable" begin
     @test sprint(print, VersionRange("0-0.3.2")) == "0-0.3.2"
@@ -133,7 +160,13 @@ temp_pkg_dir() do project_path
         Pkg.rm(TEST_PKG.name)
         @test !isinstalled(TEST_PKG)
         pkgdir = joinpath(Pkg.depots1(), "packages")
-        Pkg.gc()
+
+        # Test to ensure that with a long enough collect_delay, nothing gets reaped
+        Pkg.gc(;collect_delay=Day(1000))
+        @test !isempty(readdir(pkgdir))
+
+        # Setting collect_delay to zero causes it to be reaped immediately, howeveage
+        Pkg.gc(;collect_delay=Second(0))
         @test isempty(readdir(pkgdir))
     end
 
@@ -251,8 +284,9 @@ temp_pkg_dir() do project_path
         mktempdir() do devdir
             withenv("JULIA_PKG_DEVDIR" => devdir) do
                 try
-                    Pkg.setprotocol!(domain = "github.com", protocol = "notarealprotocol")
-                    @test_throws PkgError Pkg.develop("Example")
+                    # Test below commented out because it is really slow, https://github.com/JuliaLang/Pkg.jl/issues/1291
+                    #Pkg.setprotocol!(domain = "github.com", protocol = "notarealprotocol")
+                    #@test_throws PkgError Pkg.develop("Example")
                     Pkg.setprotocol!(domain = "github.com", protocol = "https")
                     Pkg.develop("Example")
                     @test isinstalled(TEST_PKG)
@@ -616,7 +650,7 @@ end
         # the following should not error
         Pkg.add("UUIDs")
         Pkg.test("UUIDs")
-        @test_throws PkgError("cannot `pin` stdlibs.") Pkg.pin("UUIDs")
+        @test_throws PkgError("`pin` can not be applied to `UUIDs` because it is a stdlib.") Pkg.pin("UUIDs")
     end
 end
 
@@ -706,19 +740,13 @@ end
         Pkg.generate("A")
         cd(mkdir("packages")) do
             Pkg.generate("A")
-            LibGit2.with(LibGit2.init("A")) do repo
-                LibGit2.add!(repo, "*")
-                LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
-            end
+            git_init_and_commit("A")
         end
         Pkg.generate("B")
         project = Pkg.Types.read_project("A/Project.toml")
         project.name = "B"
         Pkg.Types.write_project(project, "B/Project.toml")
-        LibGit2.with(LibGit2.init("B")) do repo
-            LibGit2.add!(repo, "*")
-            LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
-        end
+        git_init_and_commit("B")
         Pkg.develop(Pkg.PackageSpec(path = abspath("A")))
         # package with same name but different uuid exist in project
         @test_throws PkgError Pkg.develop(Pkg.PackageSpec(path = abspath("packages", "A")))
@@ -732,13 +760,10 @@ end
 @testset "issue #1180: broken toml-files in HEAD" begin
     temp_pkg_dir() do dir; cd(dir) do
         write("Project.toml", "[deps]\nExample = \n")
-        LibGit2.with(LibGit2.init(dir)) do repo
-            LibGit2.add!(repo, "*")
-            LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
-        end
+        git_init_and_commit(dir)
         write("Project.toml", "[deps]\nExample = \"7876af07-990d-54b4-ab0e-23690620f79a\"\n")
         Pkg.activate(dir)
-        @test_logs (:warn, r"Could not read project from HEAD") Pkg.status()
+        @test_logs (:warn, r"could not read project from HEAD") Pkg.status(diff=true)
     end end
 end
 
@@ -792,8 +817,26 @@ end
     end end
 end
 
+@testset "create manifest file similar to project file" begin
+    cd_tempdir() do dir
+        touch(joinpath(dir, "Project.toml"))
+        Pkg.activate(".")
+        Pkg.add("Example")
+        @test isfile(joinpath(dir, "Manifest.toml"))
+        @test !isfile(joinpath(dir, "JuliaManifest.toml"))
+    end
+    cd_tempdir() do dir
+        touch(joinpath(dir, "JuliaProject.toml"))
+        Pkg.activate(".")
+        Pkg.add("Example")
+        @test !isfile(joinpath(dir, "Manifest.toml"))
+        @test isfile(joinpath(dir, "JuliaManifest.toml"))
+    end
+end
+
 include("repl.jl")
 include("api.jl")
 include("registry.jl")
+include("artifacts.jl")
 
 end # module
