@@ -68,6 +68,10 @@ temp_pkg_dir(;rm=false) do project_path; cd(project_path) do;
     @test isinstalled(TEST_PKG)
     v = Pkg.API.__installed()[TEST_PKG.name]
     pkg"rm Example"
+    pkg"add Example, Random"
+    pkg"rm Example Random"
+    pkg"add Example,Random"
+    pkg"rm Example,Random"
     pkg"add Example#master"
 
     # Test upgrade --fixed doesn't change the tracking (https://github.com/JuliaLang/Pkg.jl/issues/434)
@@ -497,10 +501,7 @@ temp_pkg_dir() do project_path
                         Pkg.generate(pkg_name)
                     end
                     cd(pkg_name) do
-                        LibGit2.with(LibGit2.init(joinpath(project_path, parent_dir, pkg_name))) do repo
-                            LibGit2.add!(repo, "*")
-                            LibGit2.commit(repo, "initial commit"; author=TEST_SIG, committer=TEST_SIG)
-                        end
+                        git_init_and_commit(joinpath(project_path, parent_dir, pkg_name))
                     end #cd pkg_name
                 end # cd parent_dir
             end
@@ -787,6 +788,7 @@ end
         @test_throws PkgError Pkg.REPLMode.pkgstr("precompile --project")
         @test_throws PkgError Pkg.REPLMode.pkgstr("precompile Example")
         Pkg.REPLMode.pkgstr("precompile")
+        Pkg.precompile()
     end
     end
     end
@@ -890,6 +892,12 @@ end
         status Example Random
         status -m Example
         """
+        # --diff option
+        @test_logs (:warn, r"diff option only available") pkg"status --diff"
+        @test_logs (:warn, r"diff option only available") pkg"status -d"
+        git_init_and_commit(project_path)
+        @test_logs () pkg"status --diff"
+        @test_logs () pkg"status -d"
     end
 end
 
@@ -935,4 +943,13 @@ end
     end end end
 end
 
+@testset "smoke test `instantiate --verbose`" begin
+    temp_pkg_dir() do project_path
+        Pkg.activate(project_path)
+        Pkg.instantiate(verbose=true)
+        pkg"instantiate --verbose"
+        pkg"instantiate -v"
+    end
+end
+
 end # module
