@@ -1360,12 +1360,18 @@ let egal_tfunc
 end
 
 using Core.Compiler: PartialStruct, nfields_tfunc, sizeof_tfunc, sizeof_nothrow
+@test sizeof_tfunc(Const(Ptr)) === sizeof_tfunc(Union{Ptr, Int, Type{Ptr{Int8}}, Type{Int}}) === Const(Sys.WORD_SIZE ÷ 8)
+@test sizeof_tfunc(Type{Ptr}) === Int
+@test sizeof_nothrow(Union{Ptr, Int, Type{Ptr{Int8}}, Type{Int}})
+@test sizeof_nothrow(Const(Ptr))
+@test !sizeof_nothrow(Type{Ptr})
+@test !sizeof_nothrow(Type{Union{Ptr{Int}, Int}})
+@test !sizeof_nothrow(Const(Tuple))
 let PT = PartialStruct(Tuple{Int64,UInt64}, Any[Const(10, false), UInt64])
-    @test sizeof_tfunc(PT) === Const(16, false)
-    @test nfields_tfunc(PT) === Const(2, false)
-    @test sizeof_nothrow(PT) === true
+    @test sizeof_tfunc(PT) === Const(16)
+    @test nfields_tfunc(PT) === Const(2)
+    @test sizeof_nothrow(PT)
 end
-@test sizeof_nothrow(Const(Tuple)) === false
 
 using Core.Compiler: typeof_tfunc
 @test typeof_tfunc(Tuple{Vararg{Int}}) == Type{Tuple{Vararg{Int,N}}} where N
@@ -1454,6 +1460,21 @@ f_pure_add() = (1 + 1 == 2) ? true : "FAIL"
 @test Core.Compiler.getfield_tfunc(Const(Vector{Int}), Const(:mutable)) == Const(true)
 @test Core.Compiler.getfield_tfunc(DataType, Const(:mutable)) == Bool
 
+# getfield on abstract named tuples. issue #32698
+import Core.Compiler.getfield_tfunc
+@test getfield_tfunc(NamedTuple{(:id, :y), T} where {T <: Tuple{Int, Union{Float64, Missing}}},
+                     Const(:y)) == Union{Missing, Float64}
+@test getfield_tfunc(NamedTuple{(:id, :y), T} where {T <: Tuple{Int, Union{Float64, Missing}}},
+                     Const(2)) == Union{Missing, Float64}
+@test getfield_tfunc(NamedTuple{(:id, :y), T} where {T <: Tuple{Int, Union{Float64, Missing}}},
+                     Symbol) == Union{Missing, Float64, Int}
+@test getfield_tfunc(NamedTuple{<:Any, T} where {T <: Tuple{Int, Union{Float64, Missing}}},
+                     Symbol) == Union{Missing, Float64, Int}
+@test getfield_tfunc(NamedTuple{<:Any, T} where {T <: Tuple{Int, Union{Float64, Missing}}},
+                     Int) == Union{Missing, Float64, Int}
+@test getfield_tfunc(NamedTuple{<:Any, T} where {T <: Tuple{Int, Union{Float64, Missing}}},
+                     Const(:x)) == Union{Missing, Float64, Int}
+
 struct Foo_22708
     x::Ptr{Foo_22708}
 end
@@ -2421,3 +2442,6 @@ f31974(n::Int) = f31974(1:n)
 # This query hangs if type inference improperly attempts to const prop
 # call cycles.
 @test code_typed(f31974, Tuple{Int}) !== nothing
+
+f_overly_abstract_complex() = Complex(Ref{Number}(1)[])
+@test Base.return_types(f_overly_abstract_complex, Tuple{}) == [Complex]
