@@ -5,10 +5,10 @@
 end
 
 @testset "nonmissingtype" begin
-    @test Base.nonmissingtype(Union{Int, Missing}) == Int
-    @test Base.nonmissingtype(Union{Rational, Missing}) == Rational
-    @test Base.nonmissingtype(Any) == Any
-    @test Base.nonmissingtype(Missing) == Union{}
+    @test nonmissingtype(Union{Int, Missing}) == Int
+    @test nonmissingtype(Union{Rational, Missing}) == Rational
+    @test nonmissingtype(Any) == Any
+    @test nonmissingtype(Missing) == Union{}
 end
 
 @testset "convert" begin
@@ -165,6 +165,8 @@ Base.one(::Type{Unit}) = 1
         @test ismissing(f(missing))
     end
 
+    @test ismissing(clamp(missing, 1, 2))
+
     for T in (Int, Float64)
         @test zero(Union{T, Missing}) === T(0)
         @test one(Union{T, Missing}) === T(1)
